using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using mbe;
using CE3IO;

namespace jtext4mbe
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
            fontLoaded = false;
		}

        private bool fontLoaded;

		private void Form1_Load(object sender, EventArgs e)
		{
			Kst32bfont.FontPath = Properties.Settings.Default.fontFilePath;

			setTextHeight(Properties.Settings.Default.fontHeight);
			setLineWidth(Properties.Settings.Default.lineWidth);
            setWidthScale(Properties.Settings.Default.widthScale);
			mbeClipboardFormat = DataFormats.GetFormat("MbeClipboard-v1.0");
            Size sz = Size;
            MinimumSize = sz;
            sz.Width = 1000;
            MaximumSize = sz;

            if (!Kst32bfont.LoadFont()) {
                buttonPreview.Enabled = false;
                buttonCopy.Enabled = false;
                fontLoaded = false;
            } else {
                buttonPreview.Enabled = true;
                buttonCopy.Enabled = true;
                fontLoaded = true;
            }



		}

		private void buttonSet_Click(object sender, EventArgs e)
		{
			OpenFileDialog openFileDialog = new OpenFileDialog();
			openFileDialog.FileName = Properties.Settings.Default.fontFilePath;
			openFileDialog.Filter = "KST32B font file (*.txt)|*.txt";
			openFileDialog.DefaultExt = "txt";
			DialogResult dr = openFileDialog.ShowDialog();
			if (dr == DialogResult.OK) {
				Kst32bfont.FontPath = openFileDialog.FileName;
				Properties.Settings.Default.fontFilePath = openFileDialog.FileName;
                if (!Kst32bfont.LoadFont()) {
                    buttonPreview.Enabled = false;
                    buttonCopy.Enabled = false;
                    fontLoaded = false;
                } else {
                    buttonPreview.Enabled = true;
                    buttonCopy.Enabled = true;
                    fontLoaded = true;
                }
			}
		}

		private void buttonCopy_Click(object sender, EventArgs e)
		{
			Copy();			
		}

		private bool Copy()
		{
			string str = textText.Text;
			int strWidth;
			LinkedList<LineData> lineList = Kst32bfont.GetFontLineData(str, out strWidth);
			if (lineList == null) return false;

			int textHeight = getTextHeight();
			int lineWidth = getLineWidth();
            int widthScale = getWidthScale();

			LinkedList<MbeObj> objList = new LinkedList<MbeObj>();
			double scale = textHeight / 32.0;
            double dWidthScale =(double)widthScale/100;

			foreach (LineData lineData in lineList) {
				int x0 = (int)(lineData.p0.X * scale * dWidthScale);
				int y0 = (int)(lineData.p0.Y * scale);
				int x1 = (int)(lineData.p1.X * scale * dWidthScale);
				int y1 = (int)(lineData.p1.Y * scale);


				MbeObjLine objLine = new MbeObjLine();
				objLine.SetPos(new Point(x0, y0), 0);
				objLine.SetPos(new Point(x1, y1), 1);
				objLine.Layer = MbeLayer.LayerValue.PLC;
				objLine.LineWidth = lineWidth;
				objList.AddLast(objLine);
			}

			Point ptLT = new Point(0,textHeight);
			MbeDocInfo docinfo = new MbeDocInfo();

			StringBuilder strBuilder = new StringBuilder();
			StringWriter stringWriter = null;
			try {
				stringWriter = new StringWriter(strBuilder);
				WriteCE3 writeMb3 = new WriteCE3(stringWriter);
				WriteMb3(writeMb3, objList, ptLT, docinfo);
				stringWriter.Flush();
				string strBuff = strBuilder.ToString();
				System.Windows.Forms.Clipboard.SetData(mbeClipboardFormat.Name, strBuff);
			}
			catch (Exception) {
				return false;
			}
			return true;
		}

		private void WriteMb3(WriteCE3 writeMb3, LinkedList<MbeObj> mbeObjList, Point origin, MbeDocInfo docinfo)
		{
			Point offset = new Point(-origin.X, -origin.Y);
			writeMb3.WriteRecord("+MBE_DATA_V.1.0");
			writeMb3.WriteNewLine();
			docinfo.WrMb3(writeMb3);
			foreach (MbeObj obj in mbeObjList) {
				if (obj.DeleteCount < 0) {
					obj.WrMb3(writeMb3, origin);
				}
			}
			writeMb3.WriteRecord("-MBE_DATA_V.1.0");
			writeMb3.WriteNewLine();
		}



		private void buttonPreview_Click(object sender, EventArgs e)
		{
			string str = textText.Text;
			int strWidth;
			LinkedList<LineData> lineList = Kst32bfont.GetFontLineData(str,out strWidth);
			panelPreview.textHeight = getTextHeight();
			panelPreview.lineWidth = getLineWidth();
            int widthScale = getWidthScale();
            panelPreview.widthScalse = widthScale;
			panelPreview.lineDataList = lineList;
            panelPreview.AutoScrollMinSize = new Size((int)((strWidth * panelPreview.drawScale * widthScale)/100)+50,0);
			panelPreview.Invalidate();
		}

		private void buttonExit_Click(object sender, EventArgs e)
		{
			Close();
		}

		private void Form1_Closing(object sender, FormClosingEventArgs e)
		{
			Properties.Settings.Default.Save();
			getTextHeight();
			getLineWidth();
		}

        public const int MIN_WIDTH_SCALE = 50;
        public const int MAX_WIDTH_SCALE = 200;
        public const int DEFAULT_WIDTH_SCALE = 100;
        public const int MIN_LINE_WIDTH = 1500;
		public const int MAX_LINE_WIDTH = 10000;
		public const int DEFAULT_LINE_WIDTH = 2540;	//10mil
		public const int MIN_TEXT_HEIGHT = 15000;
		public const int MAX_TEXT_HEIGHT = 100000;
		public const int DEFAULT_TEXT_HEIGHT = 20000;

		private int getTextHeight()
		{
			double d;
			bool err = false; 

			string str = textSize.Text;
			try {
				d = Convert.ToDouble(str);
			}
			catch (Exception) {
				d = DEFAULT_TEXT_HEIGHT / 10000.0;
				err = true;
			}
            int n = (int)Math.Round(d * 10000);

			if (n > MAX_TEXT_HEIGHT) {
				n = MAX_TEXT_HEIGHT;
				err = true;
			} else if (n < MIN_TEXT_HEIGHT) {
				n = MIN_TEXT_HEIGHT;
				err = true;
			}
			if (err) {
				setTextHeight(n);
			}
			Properties.Settings.Default.fontHeight = n;
			return n;
		}

		private void setTextHeight(int n)
		{
			textSize.Text = String.Format("{0:##0.0###}", n / 10000.0);
		}

		private int getLineWidth()
		{
			double d;
			bool err = false;

			string str = textWidth.Text;
			try {
				d = Convert.ToDouble(str);
			}
			catch (Exception) {
				d = DEFAULT_LINE_WIDTH / 10000.0;
				err = true;
			}
			int n = (int)Math.Round(d * 10000);

			if (n > MAX_LINE_WIDTH) {
				n = MAX_LINE_WIDTH;
				err = true;
			} else if (n < MIN_LINE_WIDTH) {
				n = MIN_LINE_WIDTH;
				err = true;
			}
			if (err) {
				setLineWidth(n);
			}
			Properties.Settings.Default.lineWidth = n;
			return n;
		}

		private void setLineWidth(int n)
		{
			textWidth.Text = String.Format("{0:##0.0###}", n / 10000.0);
		}

        private int getWidthScale()
        {
            int n;
            bool err = false;

            string str = textScale.Text;
            try {
                n = Convert.ToInt32(str);
            }
            catch (Exception) {
                n = DEFAULT_WIDTH_SCALE;
                err = true;
            }
 
            if (n > MAX_WIDTH_SCALE) {
                n = MAX_WIDTH_SCALE;
                err = true;
            } else if (n < MIN_WIDTH_SCALE) {
                n = MIN_WIDTH_SCALE;
                err = true;
            }
            if (err) {
                setWidthScale(n);
            }
            Properties.Settings.Default.widthScale = n;
            return n;
        }


        private void setWidthScale(int n)
        {
            textScale.Text = n.ToString();
        }


		private DataFormats.Format mbeClipboardFormat;

		private void buttonAbout_Click(object sender, EventArgs e)
		{
			AboutBox dlg = new AboutBox();
			dlg.ShowDialog();
		}

        private void Form1_Shown(object sender, EventArgs e)
        {
            if (!fontLoaded) {
                MessageBox.Show("gpɂȂOKST32B Version 2.0tHg̃pXݒ肵Ă\n"+
                "ڂReadme.txt","jtext4mbe");
            }
        }

	}
}